﻿#region Smart-Tech (c) 2016
//=============================================================================
// This file is part of the example provided for the Smart-RPC.net library. You 
// can distribute it under the terms of the GNU General Public License as 
// published by the Free Software Foundation, version 3 of the License.
//
// This example and the Smart-RPC.library are distributed in the hope that it 
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General 
// Public License for more details.
// 
// See <http://www.gnu.org/licenses/> for a copy of the GNU General Public 
// License.
// 
// This disclaimer should remain unchanged.
//=============================================================================
#endregion

using Shared;
using SmartTech.SmartComponents.Operational;
using SmartTech.SmartComponents.SmartRpc;
using System;

namespace Client
{
    /// <summary>
    /// This class is used to receive the callbacks originating from the 
    /// service. In this case when the Add method has been called.
    /// </summary>
    public class CallbackReceiver : ICallbackContract
    {
        // All class constructors, initialize, terminate and dispose methods.
        #region Construct / Init / Terminate / Dispose

        /// <summary>
        /// Default constructor for class.
        /// </summary>
        /// <param name="name">This name is used to identify the receiver at 
        /// which the callback is received.</param>
        public CallbackReceiver(string name)
        {
            Name = name;
        }

        #endregion

        // Use this region to implement interface members for the given interface.
        #region ICallbackContract Members

        /// <summary>
        /// Inform the subscribers that integers were added.
        /// </summary>
        /// <param name="args">The argument for this callback.</param>
        public void OnIntegersAdded(CallbackArgs<Int64> args)
        {
            Logger.Log(
                EnumSeverity.Info,
                "Received notification OnIntegersAdded at receiver [{0}] with value [{1}]",
                Name,
                args.Parameter);
        }

        #endregion

        // Properties for this class.
        #region Properties

        /// <summary>
        /// This name is used to identify the receiver at which the callback is 
        /// received.
        /// </summary>
        public string Name
        {
            get;
            private set;
        }

        #endregion

        // Events for this class.
        #region Events
        #endregion

        // All public methods for this class.
        #region Methods
        #endregion

        // All protected / private methods for this class.
        #region Helper methods
        #endregion

        // All event handlers for this class.
        #region Event handlers
        #endregion

        // Private fields
        #region Fields
        #endregion
    }
}
