﻿#region Smart-Tech (c) 2016
//=============================================================================
// This file is part of the example provided for the Smart-RPC.net library. You 
// can distribute it under the terms of the GNU General Public License as 
// published by the Free Software Foundation, version 3 of the License.
//
// This example and the Smart-RPC.library are distributed in the hope that it 
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General 
// Public License for more details.
// 
// See <http://www.gnu.org/licenses/> for a copy of the GNU General Public 
// License.
// 
// This disclaimer should remain unchanged.
//=============================================================================
#endregion

using Shared;
using SmartTech.SmartComponents.Operational;
using SmartTech.SmartComponents.SmartRpc;
using System;
using System.Collections.ObjectModel;

namespace Client
{
    /// <summary>
    /// This implementation of a service adapter provides insight in how to 
    /// provide access to a service from the side of the client.
    /// </summary>
    public class ExampleServiceAdapter : ServiceAdapterBase
    {
        // All class constructors, initialize, terminate and dispose methods.
        #region Construct / Init / Terminate / Dispose

        /// <summary>
        /// Default constructor for class
        /// </summary>
        public ExampleServiceAdapter(ClientManagerBase clientManager)
            : base(clientManager)
        {
        }

        #endregion

        // Use this region to implement interface members for the given interface.
        #region <Interface> Members
        #endregion

        // Properties for this class.
        #region Properties
        #endregion

        // Events for this class.
        #region Events
        #endregion

        // All public methods for this class.
        #region Methods

        /// <summary>
        /// Have the 2 integers which are passed be added by the service.
        /// </summary>
        /// <param name="a">Integer 1.</param>
        /// <param name="b">Integer 2.</param>
        /// <returns>The result of the add.</returns>
        public ParameterResponse<EnumServiceResult, Int64> Add(Int64 a, Int64 b)
        {
            ParameterResponse<EnumServiceResult, Int64> response =
                new ParameterResponse<EnumServiceResult, Int64>(
                    EnumServiceResult.PreconditionsFailed, 0);

            response.Result = CallRemoteMethod(() =>
            {
                response = ClientManager.RequestFunction<Int64>(
                    TargetService, "Add", a, b);

                return response.Result;
            }, GetNameOfCurrentMethod());

            return response;
        }

        public ParameterResponse<EnumServiceResult, Double> Subtract(Double a, Double b)
        {
            ParameterResponse<EnumServiceResult, Double> response = 
                new ParameterResponse<EnumServiceResult, Double>(
                    EnumServiceResult.PreconditionsFailed, 0);

            response.Result = CallRemoteMethod(() =>
            {
                response = ClientManager.RequestFunction<Double>(
                    TargetService, "Subtract", a, b);

                return response.Result;
            }, GetNameOfCurrentMethod());

            return response;
        }

        public ParameterResponse<EnumServiceResult, Rectangle> CreateRectangle(Single height, Single width)
        {
            ParameterResponse<EnumServiceResult, Rectangle> response =
                new ParameterResponse<EnumServiceResult, Rectangle>(
                    EnumServiceResult.PreconditionsFailed, null);

            response.Result = CallRemoteMethod(() =>
            {
                response = ClientManager.RequestFunction<Rectangle>(
                    TargetService, "CreateRectangle", height, width);

                return response.Result;
            }, GetNameOfCurrentMethod());

            return response;
        }

        public ParameterResponse<EnumServiceResult, Collection<Rectangle>> CreateRectangleCollection(
            Int32 numberOfRectangles)
        {
            ParameterResponse<EnumServiceResult, Collection<Rectangle>> response =
                new ParameterResponse<EnumServiceResult, Collection<Rectangle>>(
                    EnumServiceResult.PreconditionsFailed, null);

            response.Result = CallRemoteMethod(() =>
            {
                response = ClientManager.RequestFunction<Collection<Rectangle>>(
                    TargetService, "CreateRectangleCollection", numberOfRectangles);

                return response.Result;
            }, GetNameOfCurrentMethod());

            return response;
        }

        /// <summary>
        /// Have this client subcribe on the callbacks related to the 
        /// corresponding service.
        /// </summary>
        /// <returns>The result of the call.</returns>
        public EnumServiceResult Subscribe()
        {
            EnumServiceResult result = EnumServiceResult.PreconditionsFailed;

            result = CallRemoteAction(() =>
            {
                result = ClientManager.RequestAction(TargetService, "Subscribe");
            }, GetNameOfCurrentMethod());

            return result;
        }

        /// <summary>
        /// Have this client unsubcribe from the callbacks related to the 
        /// corresponding service.
        /// </summary>
        /// <returns>The result of the call.</returns>
        public EnumServiceResult Unsubscribe()
        {
            EnumServiceResult result = EnumServiceResult.PreconditionsFailed;

            result = CallRemoteAction(() =>
            {
                result = ClientManager.RequestAction(TargetService, "Unsubscribe");
            }, GetNameOfCurrentMethod());

            return result;
        }

        #endregion

        // All protected / private methods for this class.
        #region Helper methods
        #endregion

        // All event handlers for this class.
        #region Event handlers
        #endregion

        // Private fields
        #region Fields

        /// <summary>
        /// The name of the service which we are targeting with this service adapter.
        /// </summary>
        private string TargetService = "ExampleService";

        #endregion
    }
}
