﻿#region Smart-Tech (c) 2016
//=============================================================================
// This file is part of the example provided for the Smart-RPC.net library. You 
// can distribute it under the terms of the GNU General Public License as 
// published by the Free Software Foundation, version 3 of the License.
//
// This example and the Smart-RPC.library are distributed in the hope that it 
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General 
// Public License for more details.
// 
// See <http://www.gnu.org/licenses/> for a copy of the GNU General Public 
// License.
// 
// This disclaimer should remain unchanged.
//=============================================================================
#endregion

using Shared;
using SmartTech.SmartComponents.Communication;
using SmartTech.SmartComponents.Operational;
using SmartTech.SmartComponents.SmartRpc;
using System;
using System.Net;
using System.Net.Sockets;

namespace Example
{
    class Program
    {
        static void Main(string[] args)
        {
            SmartTech.SmartComponents.Operational.Logger.Connect(ConsoleLogger.Instance);

            // We need to provide the local IP address since it is not possible 
            // to retrieve in the framework due to the reduced framework and the 
            // missing of required calls.
            IPAddress localIpAddress = IPAddressHelper.GetLocalIPAddress();

            if (localIpAddress != null)
            {
                // The RpcServiceHost is used to host the service(s).
                RpcServiceHost serviceHost = new RpcServiceHost();

                ExampleService exampleService = new ExampleService();
                exampleService.RegisterAsService(serviceHost);

                // Configure the broadcasts of the endpoints provided by this service. 
                // Note that the used message converter (and possibly any registered 
                // message processors) should match that of the client side !
                // Also note that the IP address used here should be updated to reflect 
                // your local ip range and subnet mask. In most cases it should be 
                // sufficient to change the first part into the same as your ip address 
                // and only replace the last part with 255. So xx.xx.xx.255.
                string firstPartOfAddress = localIpAddress.ToString()
                    .Substring(0, localIpAddress.ToString().LastIndexOf("."));
                
                IPEndPoint endPoint = new IPEndPoint(IPAddress.Parse(firstPartOfAddress + ".255"), 9999);
                UdpBroadcastSender broadcast =
                    new UdpBroadcastSender(
                        new SerializableMessageConverter(), 
                        endPoint);

                serviceHost.ConfigureServiceDiscovery(broadcast);
            }

            Console.ReadLine();
        }
    }
}
