﻿#region Smart-Tech (c) 2016
//=============================================================================
// This file is part of the example provided for the Smart-RPC.net library. You 
// can distribute it under the terms of the GNU General Public License as 
// published by the Free Software Foundation, version 3 of the License.
//
// This example and the Smart-RPC.library are distributed in the hope that it 
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General 
// Public License for more details.
// 
// See <http://www.gnu.org/licenses/> for a copy of the GNU General Public 
// License.
// 
// This disclaimer should remain unchanged.
//=============================================================================
#endregion

using SmartTech.SmartComponents.Operational;
using SmartTech.SmartComponents.SmartRpc;
using System;
using System.IO;
using System.IO.Compression;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;

namespace Shared
{
    /// <summary>
    /// This provides a way to get the local IP address so that the endpoints 
    /// provided by the example service can be correctly registered. This address 
    /// is also used to correctly broadcast the discovery information.
    /// </summary>
    public static class IPAddressHelper
    {
        // All class constructors, initialize, terminate and dispose methods.
        #region Construct / Init / Terminate / Dispose
        #endregion

        // Use this region to implement interface members for the given interface.
        #region <interface name> Members
        #endregion

        // Properties for this class.
        #region Properties
        #endregion

        // Events for this class.
        #region Events
        #endregion

        // All public methods for this class.
        #region Methods

        /// <summary>
        /// Get the local IP address.
        /// </summary>
        /// <returns></returns>
        public static IPAddress GetLocalIPAddress()
        {
            IPAddress localIPAddress = null;

            foreach (NetworkInterface item in NetworkInterface.GetAllNetworkInterfaces())
            {
                if ((item.NetworkInterfaceType == NetworkInterfaceType.Ethernet) &&
                    (item.OperationalStatus == OperationalStatus.Up))
                {
                    foreach (UnicastIPAddressInformation ip in item.GetIPProperties().UnicastAddresses)
                    {
                        // We are only interested in an actual network address and 
                        // we have to filter out the link-local (169.254.x.x) addresses.
                        if ((ip.Address.AddressFamily == AddressFamily.InterNetwork) &&
                            (ip.Address.GetAddressBytes()[0] != 169) &&
                            (ip.Address.GetAddressBytes()[1] != 254))
                        {
                            localIPAddress = ip.Address;
                            break;
                        }
                    }
                }
            }

            return localIPAddress;
        }

        #endregion

        // All protected / private methods for this class.
        #region Helper methods
        #endregion

        // All event handlers for this class.
        #region Event handlers
        #endregion

        // Private fields
        #region Fields
        #endregion
    }
}
