﻿#region Smart-Tech (c) 2016
//=============================================================================
// This file is part of the example provided for the Smart-RPC.net library. You 
// can distribute it under the terms of the GNU General Public License as 
// published by the Free Software Foundation, version 3 of the License.
//
// This example and the Smart-RPC.library are distributed in the hope that it 
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General 
// Public License for more details.
// 
// See <http://www.gnu.org/licenses/> for a copy of the GNU General Public 
// License.
// 
// This disclaimer should remain unchanged.
//=============================================================================
#endregion

using SmartTech.SmartComponents.Operational;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace Shared
{
    /// <summary>
    /// This type is used to showcase the ISerializable interface in which enables 
    /// the possibility to have custom types move across the boundary between client 
    /// and service(s). This can be used for clients without access to the other 
    /// MessageConverter solutions.
    /// </summary>
    [DataContract]
    public class Rectangle : SmartTech.SmartComponents.Operational.ISerializable
    {
        // All class constructors, initialize, terminate and dispose methods.
        #region Construct / Init / Terminate / Dispose

        public Rectangle() : this(0, 0)
        {
        }

        /// <summary>
        /// Default constructor for class
        /// </summary>
        public Rectangle(Double height, Double width)
        {
            Height = height;
            Width = width;
        }

        #endregion

        // Use this region to implement interface members for the given interface.
        #region ISerializable Members

        /// <summary>
        /// Serialize the necessary data of this instance to the passed 
        /// binarywriter so that it can be persisted and eventually be 
        /// deserialized.
        /// </summary>
        /// <param name="binaryWriter">The binary writer which needs to be 
        /// used to serialize the data to.</param>
        /// <returns><b>true</b> to indicate success, <b>false</b> otherwise.
        /// </returns>
        public bool Serialize(BinaryWriter binaryWriter)
        {
            bool result = binaryWriter.Write(Height);

            if (result)
            {
                result = binaryWriter.Write(Width);
            }

            return result;
        }

        /// <summary>
        /// Deserialize the state of this instance from the passed binary 
        /// reader.
        /// </summary>
        /// <param name="binaryReader">The binary reader from which the instance 
        /// data needs to be read.</param>
        /// <returns><b>true</b> to indicate success, <b>false</b> otherwise.
        /// </returns>
        public bool Deserialize(BinaryReader binaryReader)
        {
            Double height;
            bool result = binaryReader.Read(out height);

            if (result)
            {
                Height = height;

                Double width;
                result = binaryReader.Read(out width);

                if (result)
                {
                    Width = width;
                }
            }

            return result;
        }

        #endregion

        // Properties for this class.
        #region Properties

        /// <summary>
        /// The height of the rectangle.
        /// </summary>
        [DataMember]
        public Double Height
        {
            get;
            protected set;
        }

        /// <summary>
        /// The width of the rectangle.
        /// </summary>
        [DataMember]
        public Double Width
        {
            get;
            protected set;
        }

        #endregion

        // Events for this class.
        #region Events
        #endregion

        // All public methods for this class.
        #region Methods

        /// <summary>
        /// This is used to display the rectangle information when it is received 
        /// at the client.
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return string.Format("Rectangle({0}, {1})", Height, Width);
        }

        #endregion

        // All protected / private methods for this class.
        #region Helper methods
        #endregion

        // All event handlers for this class.
        #region Event handlers
        #endregion

        // Private fields
        #region Fields
        #endregion
    }
}
